;Software rendering example for NeoGeo CD
;furrtek 12/2011 - for neogeodev.org

    cpu 68000

    supmode on

    INCLUDE "regdefs.asm"

COMPILEFORCD SET 1


FLAG_DOTASKS         equ 0                  ;Execute la liste des taches VBL

                     org RAMSTART

FLAGS:               ds.b 1                    ;Flags gnraux (voir FLAG_*)
FRAME:               ds.w 1                    ;Dmarre  partir de la synchro audio
RANDOM32:            ds.l 1
UPBUFFER:            ds.b 2
DEST:                ds.l 1
XPEN:                ds.w 1
YPEN:                ds.w 1
XSPD:                ds.w 1
YSPD:                ds.w 1
COLOR:               ds.b 1

    IF COMPILEFORCD=1
       INCLUDE "cdheader.asm"
    ELSEIF
       INCLUDE "header.asm"
    ENDIF

VBlank:
    btst    #7,BIOS_SYSTEM_MODE
    bne     .getvbl
    jmp     BIOSF_SYSTEM_INT1
.getvbl:
    movem.l d0-d7/a0-a6,-(a7)
    move.w  #4,REG_IRQACK
    move.b  d0,REG_DIPSW
    jsr     BIOSF_SYSTEM_IO
    
    btst.b  #FLAG_DOTASKS,FLAGS
    beq     .novbl

    addq.w  #1,FRAME            ;Color change is done every 5 frames
    cmp.w   #5,FRAME
    bne     .f
    clr.w   FRAME
    cmp.b   #$FF,COLOR          ;Loop through colors 1~15
    bne     .ok
    move.b  #$11,COLOR
    bra     .f
.ok:
    addi.b  #$11,COLOR
.f:

    lea     UPBUFFER,a0         ;2*2 pixel data to be copied to DRAM
    move.b  COLOR,(a0)+
    move.b  COLOR,(a0)

    clr.l   d0                  ;Calc Y position in DRAM
    clr.l   d1
    move.w  YPEN,d0
    asr.w   #4,d0               ;12.4 to integer
    andi.w  #3,d0
    lsl.w   #1,d0               ;Inside tile
    move.w  YPEN,d1
    asr.w   #4,d1               ;12.4 to integer
    andi.w  #$FFFC,d1
    lsl.w   #3,d1               ;Break out of tile boundary
    add.w   d1,d0
    move.l  d0,DEST
    add.l   d0,DEST

    clr.l   d1                  ;Calc X position in DRAM
    move.w  XPEN,d0
    asr.w   #4,d0               ;12.4 to integer
    andi.w  #3,d0
    lea     tbl,a0
    move.b  0(a0,d0),d1
    lsl.w   #3,d1               ;Inside tile
    move.w  XPEN,d0
    asr.w   #4,d0               ;12.4 to integer
    andi.w  #$FFFC,d0
    lsl.l   #8,d0               ;Break out of tile boundary
    add.l   d1,d0
    add.l   d0,DEST
    add.l   d0,DEST

    addi.l  #64*(32+2),DEST     ;First visible tile address

    move.l  #UPBUFFER,BIOS_UPSRC
    move.l  DEST,BIOS_UPDEST
    move.l  #1,BIOS_UPSIZE      ;Will write two bytes
    move.b  #1,BIOS_UPZONE      ;FIX zone
    move.b  #0,BIOS_UPBANK

    jsr     BIOSF_UPLOAD

    move.w  XSPD,d0             ;Update positions
    add.w   d0,XPEN
    move.w  YSPD,d0
    add.w   d0,YPEN

    cmp.w  #140<<4,XPEN         ;Randomize speed on bounces
    blo    .noright
    jsr    getrandspeed
    not    d0
    move.w d0,XSPD
.noright:
    cmp.w  #10<<4,XPEN
    bhi    .noleft
    jsr    getrandspeed
    move.w d0,XSPD
.noleft:

    cmp.w  #100<<4,YPEN
    blo    .nobot
    jsr    getrandspeed
    not    d0
    move.w d0,YSPD
.nobot:
    cmp.w  #10<<4,YPEN
    bhi    .notop
    jsr    getrandspeed
    move.w d0,YSPD
.notop:

.novbl:
    movem.l (a7)+,d0-d7/a0-a6
    rte

;Conversion from x to tile column
tbl:
   dc.b 2,3,0,1

;Get random speed between 5 and 12
getrandspeed:
    jsr    GALOISPRN
    move.l RANDOM32,d0
    andi.w #7,d0
    addi.w #5,d0
    rts

RLI:
    move.w  #2,REG_IRQACK
    rte

IRQ3:
    move.w  #1,REG_IRQACK
    rte

Start:
    lea     $10F300,a7
    move.b  d0,REG_DIPSW
    move.w  #7,REG_IRQACK
    clr.b   FLAGS
    move.w  #$2000,sr         ; Activate interrupts

    move.l  #($F2FF/4)-1,d7   ; Clear user ram
    lea     RAMSTART,a0
clram:
    clr.l   (a0)+
    dbra    d7,clram

    jsr     BIOSF_LSP_1ST     ; Clear sprites and tilemap
    jsr     BIOSF_FIX_CLEAR

    lea     Pal_rainbow,a0    ; Load palette
    lea     PALETTES,a1
    move.l  #16-1,d7
ldpal:
    move.w  (a0)+,(a1)+
    dbra    d7,ldpal
    
    move.w  #$8000,BACKDROPCOLOR
    
    move.w  #FIXMAP,VRAM_ADDR    ;Map tiles 0~4FF to get a "bitmap" to draw to
    move.w  #1,VRAM_MOD
    move.l  #(40*32)-1,d7
    clr.l   d0
mp:
    move.w  d0,VRAM_RW
    addi.w  #1,d0
    dbra    d7,mp

    move.l  #$13371337,RANDOM32  ;PRN seed

    move.w  #0,REG_LSPCMODE

    move.w  #8,XSPD              ;Initial speed and position of pen
    move.w  #9,YSPD
    move.w  #100<<4,XPEN
    move.w  #60<<4,YPEN

    bset.b  #FLAG_DOTASKS,FLAGS

Loop:
    bra     Loop

Pal_rainbow:
    dc.w $0000,$6DB0,$2B90,$4870
    dc.w $0650,$2430,$6110,$0000
    dc.w $0000,$6110,$2430,$0650
    dc.w $4870,$2B90,$6DB0,$6EC0

GaloisPRN:
    movem.l d0-d2,-(a7)
    move.l  #$AF-$100,d1
    move.l  #18,d2
    move.l  RANDOM32,d0
Ninc0:
    add.l   d0,d0
    bcc     Ninc1
    eor.b   d1,d0
Ninc1:
    dbra    d2,Ninc0
    move.l  d0,RANDOM32
    movem.l (a7)+,d0-d2
    rts
